# bharat\states\states.py

import json
from pathlib import Path
from typing import List

from .objects import Date, State

STATES_JSON_FILE = Path(__file__).parent.joinpath('states.json')   # All states data stored here
with open(STATES_JSON_FILE) as states_list:
    list_state_ut_dict = json.load(states_list)          # Is a list[dict]


ALL: List[State] = [None] * len(list_state_ut_dict)      # Should be a list[State]
__all__: List[str] = [None] * len(list_state_ut_dict)    # Module info


for i, state in enumerate(list_state_ut_dict):
    state["statehood_date"] = Date(state["statehood_date"])
    exec(f'{state["abbr"]} = State(**state)')

    ALL[i] = eval(f'{state["abbr"]}')
    __all__[i] = state["abbr"]


STATES: List[State] = [state for state in ALL if not state.is_UT]
UNION_TERRITORIES: List[State] = [state for state in ALL if state.is_UT]

__all__.extend(["STATES", "UNION_TERRITORIES", "ALL"])



# bharat\states\objects.py

import datetime


class Date(datetime.date):
	"""
	Overrides datetime.date object's __repr__ mathod

	Only purpose is the nice representation of a date
	"""

	def __new__(self, date_str):
		# The date is stored as YYYY-MM-DD in the JSON file
		return datetime.date.__new__(self, *map(int, date_str.split('-')))

	def __repr__(self) -> str:
		return self.strftime('%d-%m-%Y')


class State:
	"""
	State class:
	================
	A state has the following attributes:
		- abbr				: State code (string)
		- code				: GST code (integer)
		- capital			: Capital(s) [comma separated if multiple]
		- iso31662in			: ISO31662IN code (string)
		- is_UT				: True if the state is a union territory else False
		- name				: Name of state
		- post_zone			: Postal zone (integer)
		- statehood_date		: Date of statehood (date object)
		- website			: State's website
		- zone				: Geographical zone (string)
	"""

	abbr: str
	capital: str
	code: int
	iso31662in: str
	is_UT: bool
	name: str
	post_zone: int
	statehood_date: Date
	website: str
	zone: str

	def __init__(self, **kwargs):
		self.__dict__.update(kwargs)

	def __repr__(self) -> str:
		if self.is_UT:
			return f"<Union Territory:{self.name}>"
		else:
			return f"<State:{self.name}>"

	def __str__(self) -> str:
		return self.name



from setuptools import setup, find_packages

long_description = open("README.md").read()

setup(
	name="bharat",
	version="0.1",
	author="Shraddha Kishan",
	author_email="shraddha.kishan@gmail.com",
	url="https://github.com/bytebaker/bharat",
	description="India states metadata",
	long_description=long_description,
	long_description_content_type="text/markdown",
	license="BSD",
	packages=find_packages(),
	include_package_data=True,
	install_requires=[],
	entry_points={},
	platforms=["any"],
	classifiers=[
		"Programming Language :: Python :: 3"
	],
)